@@ -27,8 +27,7 @@
 
 /**
  * SECTION:clutter-alpha
- * @short_description: A class for calculating an alpha value as a function
- * of time.
+ * @short_description: A class for calculating a value as a function of time
  *
  * #ClutterAlpha is a class for calculating an floating point value
  * dependent only on the position of a #ClutterTimeline.
@@ -77,17 +76,17 @@
  *     function called my_sine_alpha(). The defined #ClutterAlpha
  *     instance can be reused in multiple #ClutterBehaviour
  *     definitions or for #ClutterAnimation definitions.</para>
- *     <programlisting>
- *  {
- *    "id" : "sine-alpha",
- *    "timeline" : {
- *      "id" : "sine-timeline",
- *      "duration" : 500,
- *      "loop" : true
- *    },
- *    "function" : "my_sine_alpha"
- *  }
- *     </programlisting>
+ *     <programlisting><![CDATA[
+ * {
+ *   "id" : "sine-alpha",
+ *   "timeline" : {
+ *     "id" : "sine-timeline",
+ *     "duration" : 500,
+ *     "loop" : true
+ *   },
+ *   "function" : "my_sine_alpha"
+ * }
+ *    ]]></programlisting>
  *   </example>
  *   <para>For the way to define the #ClutterAlpha:mode property
  *   inside a ClutterScript fragment, see <link
@@ -333,8 +332,8 @@ clutter_alpha_class_init (ClutterAlphaClass *klass)
    * Since: 0.2
    */
   pspec = g_param_spec_object ("timeline",
-                               "Timeline",
-                               "Timeline used by the alpha",
+                               P_("Timeline"),
+                               P_("Timeline used by the alpha"),
                                CLUTTER_TYPE_TIMELINE,
                                CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_TIMELINE, pspec);
@@ -349,8 +348,8 @@ clutter_alpha_class_init (ClutterAlphaClass *klass)
    * Since: 0.2
    */
   pspec = g_param_spec_double ("alpha",
-                               "Alpha value",
-                               "Alpha value",
+                               P_("Alpha value"),
+                               P_("Alpha value as computed by the alpha"),
                                -1.0, 2.0,
                                0.0,
                                CLUTTER_PARAM_READABLE);
@@ -370,8 +369,8 @@ clutter_alpha_class_init (ClutterAlphaClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_ulong ("mode",
-                              "Mode",
-                              "Progress mode",
+                              P_("Mode"),
+                              P_("Progress mode"),
                               0, G_MAXULONG,
                               CLUTTER_CUSTOM_MODE,
                               G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE);
@@ -459,7 +458,7 @@ clutter_alpha_set_closure_internal (ClutterAlpha *alpha,
   /* set the marshaller */
   if (G_CLOSURE_NEEDS_MARSHAL (closure))
     {
-      GClosureMarshal marshal = clutter_marshal_DOUBLE__VOID;
+      GClosureMarshal marshal = _clutter_marshal_DOUBLE__VOID;
 
       g_closure_set_marshal (closure, marshal);
     }
