@@ -85,6 +85,12 @@ struct _ClutterBackendClass
                                                 ClutterStage    *stage);
   ClutterDeviceManager *(* get_device_manager) (ClutterBackend  *backend);
 
+  void                  (* copy_event_data)    (ClutterBackend  *backend,
+                                                ClutterEvent    *src,
+                                                ClutterEvent    *dest);
+  void                  (* free_event_data)    (ClutterBackend  *backend,
+                                                ClutterEvent    *event);
+
   /* signals */
   void (* resolution_changed) (ClutterBackend *backend);
   void (* font_changed)       (ClutterBackend *backend);
