@@ -21,9 +21,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -40,6 +38,10 @@
  * Since: 0.4
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "clutter-fixed.h"
 #include "clutter-marshal.h"
 #include "clutter-behaviour-ellipse.h"
@@ -50,12 +52,6 @@
 #include <stdlib.h>
 #include <memory.h>
 
-/****************************************************************************
- *                                                                          *
- * ClutterBehaviourEllipse                                                  *
- *                                                                          *
- ****************************************************************************/
-
 G_DEFINE_TYPE (ClutterBehaviourEllipse,
                clutter_behaviour_ellipse,
 	       CLUTTER_TYPE_BEHAVIOUR);
@@ -390,8 +386,8 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_double ("angle-start",
-                               "Start Angle",
-                               "Initial angle",
+                               P_("Start Angle"),
+                               P_("Initial angle"),
                                0.0, 360.0,
                                0.0,
                                CLUTTER_PARAM_READWRITE);
@@ -405,8 +401,8 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_double ("angle-end",
-                               "End Angle",
-                               "Final angle",
+                               P_("End Angle"),
+                               P_("Final angle"),
                                0.0, 360.0,
                                0.0,
                                CLUTTER_PARAM_READWRITE);
@@ -420,8 +416,8 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_double ("angle-tilt-x",
-                               "Angle x tilt",
-                               "Tilt of the ellipse around x axis",
+                               P_("Angle x tilt"),
+                               P_("Tilt of the ellipse around x axis"),
                                0.0, 360.0,
                                360.0,
                                CLUTTER_PARAM_READWRITE);
@@ -435,8 +431,8 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_double ("angle-tilt-y",
-                               "Angle y tilt",
-                               "Tilt of the ellipse around y axis",
+                               P_("Angle y tilt"),
+                               P_("Tilt of the ellipse around y axis"),
                                0.0, 360.0,
                                360.0,
                                CLUTTER_PARAM_READWRITE);
@@ -450,8 +446,8 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_double ("angle-tilt-z",
-                               "Angle z tilt",
-                               "Tilt of the ellipse around z axis",
+                               P_("Angle z tilt"),
+                               P_("Tilt of the ellipse around z axis"),
                                0.0, 360.0,
                                360.0,
                                CLUTTER_PARAM_READWRITE);
@@ -465,8 +461,8 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_int ("width",
-                            "Width",
-                            "Width of the ellipse",
+                            P_("Width"),
+                            P_("Width of the ellipse"),
                             0, G_MAXINT,
                             100,
                             CLUTTER_PARAM_READWRITE);
@@ -480,8 +476,8 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_int ("height",
-                            "Height",
-                            "Height of ellipse",
+                            P_("Height"),
+                            P_("Height of ellipse"),
                             0, G_MAXINT,
                             50,
                             CLUTTER_PARAM_READWRITE);
@@ -495,8 +491,8 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_boxed ("center",
-                              "Center",
-                              "Center of ellipse",
+                              P_("Center"),
+                              P_("Center of ellipse"),
                               CLUTTER_TYPE_KNOT,
                               CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_CENTER, pspec);
@@ -509,8 +505,8 @@ clutter_behaviour_ellipse_class_init (ClutterBehaviourEllipseClass *klass)
    * Since: 0.4
    */
   pspec = g_param_spec_enum ("direction",
-                             "Direction",
-                             "Direction of rotation",
+                             P_("Direction"),
+                             P_("Direction of rotation"),
                              CLUTTER_TYPE_ROTATE_DIRECTION,
                              CLUTTER_ROTATE_CW,
                              CLUTTER_PARAM_READWRITE);
