@@ -210,9 +210,9 @@ clutter_behaviour_path_class_init (ClutterBehaviourPathClass *klass)
   gobject_class->dispose = clutter_behaviour_path_dispose;
 
   pspec = g_param_spec_object ("path",
-                               "Path",
-                               "The ClutterPath object representing the path "
-                               "to animate along",
+                               P_("Path"),
+                               P_("The ClutterPath object representing the path "
+                                  "to animate along"),
                                CLUTTER_TYPE_PATH,
                                CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_PATH, pspec);
@@ -233,7 +233,7 @@ clutter_behaviour_path_class_init (ClutterBehaviourPathClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterBehaviourPathClass, knot_reached),
                   NULL, NULL,
-                  clutter_marshal_VOID__UINT,
+                  _clutter_marshal_VOID__UINT,
                   G_TYPE_NONE, 1,
                   G_TYPE_UINT);
 
