@@ -284,6 +284,7 @@ static void
 clutter_behaviour_class_init (ClutterBehaviourClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GParamSpec *pspec;
 
   object_class->dispose      = clutter_behaviour_dispose;
   object_class->set_property = clutter_behaviour_set_property;
@@ -299,13 +300,12 @@ clutter_behaviour_class_init (ClutterBehaviourClass *klass)
    *
    * Since: 0.2
    */
-  g_object_class_install_property (object_class,
-                                   PROP_ALPHA,
-                                   g_param_spec_object ("alpha",
-                                                        "Alpha",
-                                                        "Alpha Object to drive the behaviour",
-                                                        CLUTTER_TYPE_ALPHA,
-                                                        CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_object ("alpha",
+                               P_("Alpha"),
+                               P_("Alpha Object to drive the behaviour"),
+                               CLUTTER_TYPE_ALPHA,
+                               CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_ALPHA, pspec);
 
   klass->alpha_notify = clutter_behaviour_alpha_notify_unimplemented;
 
@@ -325,7 +325,7 @@ clutter_behaviour_class_init (ClutterBehaviourClass *klass)
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (ClutterBehaviourClass, applied),
 		  NULL, NULL,
-		  clutter_marshal_VOID__OBJECT,
+		  _clutter_marshal_VOID__OBJECT,
 		  G_TYPE_NONE, 1,
 		  CLUTTER_TYPE_ACTOR);
   /**
@@ -336,6 +336,7 @@ clutter_behaviour_class_init (ClutterBehaviourClass *klass)
    * The ::removed signal is emitted each time a behaviour is not applied
    * to an actor anymore.
    *
+   * Since: 0.4
    */
   behave_signals[REMOVED] =
     g_signal_new ("removed",
@@ -343,7 +344,7 @@ clutter_behaviour_class_init (ClutterBehaviourClass *klass)
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (ClutterBehaviourClass, removed),
 		  NULL, NULL,
-		  clutter_marshal_VOID__OBJECT,
+		  _clutter_marshal_VOID__OBJECT,
 		  G_TYPE_NONE, 1,
 		  CLUTTER_TYPE_ACTOR);
 
