@@ -81,9 +81,10 @@
  *    &ast; mask of the key event. the returned value can be directly
  *    &ast; used to signal that the actor has handled the event.
  *    &ast;/
- *   return clutter_binding_pool_activate (pool, G_OBJECT (actor),
+ *   return clutter_binding_pool_activate (pool,
  *                                         key_event-&gt;keyval,
- *                                         key_event-&gt;modifier_state);
+ *                                         key_event-&gt;modifier_state,
+ *                                         G_OBJECT (actor));
  * ]|
  *
  * The clutter_binding_pool_activate() function will return %FALSE if
@@ -312,8 +313,8 @@ clutter_binding_pool_class_init (ClutterBindingPoolClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_string ("name",
-                               "Name",
-                               "The unique name of the binding pool",
+                               P_("Name"),
+                               P_("The unique name of the binding pool"),
                                NULL,
                                CLUTTER_PARAM_READWRITE |
                                G_PARAM_CONSTRUCT_ONLY);
@@ -502,7 +503,7 @@ clutter_binding_pool_install_action (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
       g_closure_set_marshal (closure, marshal);
     }
 
@@ -568,7 +569,7 @@ clutter_binding_pool_install_closure (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
       g_closure_set_marshal (closure, marshal);
     }
 
@@ -636,7 +637,7 @@ clutter_binding_pool_override_action (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
       g_closure_set_marshal (closure, marshal);
     }
 }
@@ -696,7 +697,7 @@ clutter_binding_pool_override_closure (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
       g_closure_set_marshal (closure, marshal);
     }
 }
