@@ -124,8 +124,8 @@ clutter_input_device_class_init (ClutterInputDeviceClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_int ("id",
-                            "Id",
-                            "Unique identifier of the device",
+                            P_("Id"),
+                            P_("Unique identifier of the device"),
                             -1, G_MAXINT,
                             0,
                             CLUTTER_PARAM_READWRITE |
@@ -140,8 +140,8 @@ clutter_input_device_class_init (ClutterInputDeviceClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_string ("name",
-                               "Name",
-                               "The name of the device",
+                               P_("Name"),
+                               P_("The name of the device"),
                                NULL,
                                CLUTTER_PARAM_READWRITE |
                                G_PARAM_CONSTRUCT_ONLY);
@@ -155,8 +155,8 @@ clutter_input_device_class_init (ClutterInputDeviceClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_enum ("device-type",
-                             "Device Type",
-                             "The type of the device",
+                             P_("Device Type"),
+                             P_("The type of the device"),
                              CLUTTER_TYPE_INPUT_DEVICE_TYPE,
                              CLUTTER_POINTER_DEVICE,
                              CLUTTER_PARAM_READWRITE |
@@ -261,22 +261,43 @@ void
 _clutter_input_device_set_stage (ClutterInputDevice *device,
                                  ClutterStage       *stage)
 {
+  ClutterStage *old_stage;
+
   g_return_if_fail (CLUTTER_IS_INPUT_DEVICE (device));
 
+  old_stage = device->stage;
   device->stage = stage;
 
   /* if we left the stage then we also need to unset the
    * cursor actor (and update its :has-pointer property)
    */
-  if (device->stage == NULL && device->cursor_actor != NULL)
+  if (device->stage == NULL &&
+      device->cursor_actor != NULL &&
+      device->cursor_actor != CLUTTER_ACTOR (old_stage))
     {
+      ClutterEvent cev;
+
+      cev.crossing.type = CLUTTER_LEAVE;
+      cev.crossing.time = device->current_time;
+      cev.crossing.flags = 0;
+      cev.crossing.stage = old_stage;
+      cev.crossing.source = device->cursor_actor;
+      cev.crossing.x = device->current_x;
+      cev.crossing.y = device->current_y;
+      cev.crossing.device = device;
+      cev.crossing.related = device->stage != NULL
+                           ? CLUTTER_ACTOR (device->stage)
+                           : CLUTTER_ACTOR (old_stage);
+
+      _clutter_stage_queue_event (old_stage, &cev);
+
       _clutter_actor_set_has_pointer (device->cursor_actor, FALSE);
       g_object_weak_unref (G_OBJECT (device->cursor_actor),
                            cursor_weak_unref,
                            device);
-
-      device->cursor_actor = NULL;
     }
+
+  device->cursor_actor = NULL;
 }
 
 /*
@@ -342,11 +363,40 @@ _clutter_input_device_set_actor (ClutterInputDevice *device,
       cev.crossing.time = device->current_time;
       cev.crossing.flags = 0;
       cev.crossing.stage = device->stage;
-      cev.crossing.source = actor;
       cev.crossing.x = device->current_x;
       cev.crossing.y = device->current_y;
       cev.crossing.device = device;
-      cev.crossing.related = old_actor;
+
+      CLUTTER_NOTE (EVENT, "Device '%s' entering '%s' at %d, %d",
+                    device->device_name,
+                    clutter_actor_get_name (actor) != NULL
+                      ? clutter_actor_get_name (actor)
+                      : G_OBJECT_TYPE_NAME (actor),
+                    device->current_x,
+                    device->current_y);
+
+      /* if there is an actor overlapping the Stage boundary and we
+       * don't do this check then we'll emit an ENTER event only on
+       * the actor instead of emitting it on the Stage *and* the
+       * actor
+       */
+      if (old_actor == NULL && actor != CLUTTER_ACTOR (device->stage))
+        {
+          cev.crossing.source = CLUTTER_ACTOR (device->stage);
+          cev.crossing.related = NULL;
+
+          CLUTTER_NOTE (EVENT, "Adding Crossing[Enter] event for Stage");
+
+          _clutter_process_event (&cev);
+
+          cev.crossing.source = actor;
+          cev.crossing.related = CLUTTER_ACTOR (device->stage);
+        }
+      else
+        {
+          cev.crossing.source = actor;
+          cev.crossing.related = old_actor;
+        }
 
       /* as above: we need to make sure that this event is processed
        * before any other event we might have queued up until now, so
@@ -505,6 +555,25 @@ clutter_input_device_get_pointer_actor (ClutterInputDevice *device)
   return device->cursor_actor;
 }
 
+/**
+ * clutter_input_device_get_pointer_stage:
+ * @device: a #ClutterInputDevice of type %CLUTTER_POINTER_DEVICE
+ *
+ * Retrieves the #ClutterStage underneath the pointer of @device
+ *
+ * Return value: (transfer none): a pointer to the #ClutterStage or %NULL
+ *
+ * Since: 1.2
+ */
+ClutterStage *
+clutter_input_device_get_pointer_stage (ClutterInputDevice *device)
+{
+  g_return_val_if_fail (CLUTTER_IS_INPUT_DEVICE (device), NULL);
+  g_return_val_if_fail (device->device_type == CLUTTER_POINTER_DEVICE, NULL);
+
+  return device->stage;
+}
+
 /**
  * clutter_input_device_get_device_name:
  * @device: a #ClutterInputDevice
@@ -524,3 +593,92 @@ clutter_input_device_get_device_name (ClutterInputDevice *device)
 
   return device->device_name;
 }
+
+/**
+ * clutter_input_device_update_from_event:
+ * @device: a #ClutterInputDevice
+ * @event: a #ClutterEvent
+ * @update_stage: whether to update the #ClutterStage of the @device
+ *   using the stage of the event
+ *
+ * Forcibly updates the state of the @device using a #ClutterEvent
+ *
+ * This function should never be used by applications: it is meant
+ * for integration with embedding toolkits, like clutter-gtk
+ *
+ * Embedding toolkits that disable the event collection inside Clutter
+ * need to use this function to update the state of input devices depending
+ * on a #ClutterEvent that they are going to submit to the event handling code
+ * in Clutter though clutter_do_event(). Since the input devices hold the state
+ * that is going to be used to fill in fields like the #ClutterButtonEvent
+ * click count, or to emit synthesized events like %CLUTTER_ENTER and
+ * %CLUTTER_LEAVE, it is necessary for embedding toolkits to also be
+ * responsible of updating the input device state.
+ *
+ * For instance, this might be the code to translate an embedding toolkit
+ * native motion notification into a Clutter #ClutterMotionEvent and ask
+ * Clutter to process it:
+ *
+ * |[
+ *   ClutterEvent c_event;
+ *
+ *   translate_native_event_to_clutter (native_event, &amp;c_event);
+ *
+ *   clutter_do_event (&amp;c_event);
+ * ]|
+ *
+ * Before letting clutter_do_event() process the event, it is necessary to call
+ * clutter_input_device_update_from_event():
+ *
+ * |[
+ *   ClutterEvent c_event;
+ *   ClutterDeviceManager *manager;
+ *   ClutterInputDevice *device;
+ *
+ *   translate_native_event_to_clutter (native_event, &amp;c_event);
+ *
+ *   /&ast; get the device manager &ast;/
+ *   manager = clutter_device_manager_get_default ();
+ *
+ *   /&ast; use the default Core Pointer that Clutter
+ *    &ast; backends register by default
+ *    &ast;/
+ *   device = clutter_device_manager_get_core_device (manager, %CLUTTER_POINTER_DEVICE);
+ *
+ *   /&ast; update the state of the input device &ast;/
+ *   clutter_input_device_update_from_event (device, &amp;c_event, FALSE);
+ *
+ *   clutter_do_event (&amp;c_event);
+ * ]|
+ *
+ * The @update_stage boolean argument should be used when the input device
+ * enters and leaves a #ClutterStage; it will use the #ClutterStage field
+ * of the passed @event to update the stage associated to the input device.
+ *
+ * Since: 1.2
+ */
+void
+clutter_input_device_update_from_event (ClutterInputDevice *device,
+                                        ClutterEvent       *event,
+                                        gboolean            update_stage)
+{
+  ClutterModifierType event_state;
+  ClutterStage *event_stage;
+  gfloat event_x, event_y;
+  guint32 event_time;
+
+  g_return_if_fail (CLUTTER_IS_INPUT_DEVICE (device));
+  g_return_if_fail (event != NULL);
+
+  event_state = clutter_event_get_state (event);
+  event_time = clutter_event_get_time (event);
+  event_stage = clutter_event_get_stage (event);
+  clutter_event_get_coords (event, &event_x, &event_y);
+
+  _clutter_input_device_set_coords (device, event_x, event_y);
+  _clutter_input_device_set_state (device, event_state);
+  _clutter_input_device_set_time (device, event_time);
+
+  if (update_stage)
+    _clutter_input_device_set_stage (device, event_stage);
+}
