@@ -34,6 +34,8 @@
 
 #include <glib.h>
 
+#include <glib/gi18n-lib.h>
+
 #include "pango/cogl-pango.h"
 
 #include "clutter-backend.h"
@@ -44,6 +46,7 @@
 #include "clutter-id-pool.h"
 #include "clutter-layout-manager.h"
 #include "clutter-master-clock.h"
+#include "clutter-settings.h"
 #include "clutter-stage-manager.h"
 #include "clutter-stage-window.h"
 #include "clutter-stage.h"
@@ -53,32 +56,44 @@ G_BEGIN_DECLS
 
 typedef struct _ClutterMainContext      ClutterMainContext;
 
-typedef enum {
-  CLUTTER_ACTOR_UNUSED_FLAG    = 0,
+#define CLUTTER_PRIVATE_FLAGS(a)	 (((ClutterActor *) (a))->private_flags)
+#define CLUTTER_SET_PRIVATE_FLAGS(a,f)	 (CLUTTER_PRIVATE_FLAGS (a) |= (f))
+#define CLUTTER_UNSET_PRIVATE_FLAGS(a,f) (CLUTTER_PRIVATE_FLAGS (a) &= ~(f))
 
-  CLUTTER_ACTOR_IN_DESTRUCTION = 1 << 0,
-  CLUTTER_ACTOR_IS_TOPLEVEL    = 1 << 1,
-  CLUTTER_ACTOR_IN_REPARENT    = 1 << 2,
+#define CLUTTER_ACTOR_IS_TOPLEVEL(a)            ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IS_TOPLEVEL) != FALSE)
+#define CLUTTER_ACTOR_IS_INTERNAL_CHILD(a)      ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_INTERNAL_CHILD) != FALSE)
+#define CLUTTER_ACTOR_IN_DESTRUCTION(a)         ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_DESTRUCTION) != FALSE)
+#define CLUTTER_ACTOR_IN_REPARENT(a)            ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_REPARENT) != FALSE)
+#define CLUTTER_ACTOR_IN_PAINT(a)               ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_PAINT) != FALSE)
+#define CLUTTER_ACTOR_IN_RELAYOUT(a)            ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_RELAYOUT) != FALSE)
+#define CLUTTER_STAGE_IN_RESIZE(a)              ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_RESIZE) != FALSE)
+
+typedef enum {
+  CLUTTER_ACTOR_UNUSED_FLAG = 0,
+
+  CLUTTER_IN_DESTRUCTION = 1 << 0,
+  CLUTTER_IS_TOPLEVEL    = 1 << 1,
+  CLUTTER_IN_REPARENT    = 1 << 2,
 
   /* Used by the stage to indicate GL viewport / perspective etc needs
    * (re)setting.
    */
-  CLUTTER_ACTOR_SYNC_MATRICES  = 1 << 3,
+  CLUTTER_SYNC_MATRICES  = 1 << 3,
 
   /* Used to avoid recursion */
-  CLUTTER_ACTOR_IN_PAINT       = 1 << 4,
+  CLUTTER_IN_PAINT       = 1 << 4,
 
   /* Used to avoid recursion */
-  CLUTTER_ACTOR_IN_RELAYOUT    = 1 << 5,
+  CLUTTER_IN_RELAYOUT    = 1 << 5,
 
   /* Used by the stage if resizing is an asynchronous operation (like on
    * X11) to delay queueing relayouts until we got a notification from the
    * event handling
    */
-  CLUTTER_STAGE_IN_RESIZE      = 1 << 6,
+  CLUTTER_IN_RESIZE      = 1 << 6,
 
   /* a flag for internal children of Containers */
-  CLUTTER_ACTOR_INTERNAL_CHILD = 1 << 7
+  CLUTTER_INTERNAL_CHILD = 1 << 7
 } ClutterPrivateFlags;
 
 struct _ClutterInputDevice
@@ -171,6 +186,8 @@ struct _ClutterMainContext
   gulong redraw_count;
 
   GList *repaint_funcs;
+
+  ClutterSettings *settings;
 };
 
 #define CLUTTER_CONTEXT()	(_clutter_context_get_default ())
@@ -179,10 +196,6 @@ gboolean            _clutter_context_is_initialized (void);
 PangoContext *_clutter_context_create_pango_context (ClutterMainContext *self);
 PangoContext *_clutter_context_get_pango_context    (ClutterMainContext *self);
 
-#define CLUTTER_PRIVATE_FLAGS(a)	 (((ClutterActor *) (a))->private_flags)
-#define CLUTTER_SET_PRIVATE_FLAGS(a,f)	 (CLUTTER_PRIVATE_FLAGS (a) |= (f))
-#define CLUTTER_UNSET_PRIVATE_FLAGS(a,f) (CLUTTER_PRIVATE_FLAGS (a) &= ~(f))
-
 #define CLUTTER_PARAM_READABLE  \
         G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB
 #define CLUTTER_PARAM_WRITABLE  \
@@ -192,6 +205,15 @@ PangoContext *_clutter_context_get_pango_context    (ClutterMainContext *self);
 
 #define I_(str)  (g_intern_static_string ((str)))
 
+/* mark all properties under the "Property" context */
+#ifdef ENABLE_NLS
+#define P_(String) (_clutter_gettext ((String)))
+#else
+#define P_(String) (String)
+#endif
+
+G_CONST_RETURN gchar *_clutter_gettext (const gchar *str);
+
 /* device manager */
 void _clutter_device_manager_add_device     (ClutterDeviceManager *device_manager,
                                              ClutterInputDevice   *device);
@@ -242,7 +264,6 @@ int      _clutter_stage_get_pending_swaps      (ClutterStage *stage);
 
 gboolean _clutter_stage_has_full_redraw_queued (ClutterStage *stage);
 
-
 /* vfuncs implemented by backend */
 GType         _clutter_backend_impl_get_type  (void);
 
@@ -267,6 +288,12 @@ gboolean      _clutter_backend_post_parse     (ClutterBackend  *backend,
                                                GError         **error);
 void          _clutter_backend_init_events    (ClutterBackend  *backend);
 
+void          _clutter_backend_copy_event_data (ClutterBackend  *backend,
+                                                ClutterEvent    *src,
+                                                ClutterEvent    *dest);
+void          _clutter_backend_free_event_data (ClutterBackend  *backend,
+                                                ClutterEvent    *event);
+
 ClutterFeatureFlags _clutter_backend_get_features (ClutterBackend *backend);
 
 gfloat        _clutter_backend_get_units_per_em   (ClutterBackend       *backend,
@@ -338,6 +365,10 @@ void     _clutter_effect_post_paint (ClutterEffect *effect);
 
 GType _clutter_layout_manager_get_child_meta_type (ClutterLayoutManager *manager);
 
+void     _clutter_event_set_platform_data (ClutterEvent       *event,
+                                           gpointer            data);
+gpointer _clutter_event_get_platform_data (const ClutterEvent *event);
+
 G_END_DECLS
 
 #endif /* _HAVE_CLUTTER_PRIVATE_H */
