@@ -43,6 +43,11 @@ G_BEGIN_DECLS
  * blended together.
  */
 
+typedef struct _CoglMaterial	      CoglMaterial;
+typedef struct _CoglMaterialLayer     CoglMaterialLayer;
+
+#define COGL_MATERIAL(OBJECT) ((CoglMaterial *)OBJECT)
+
 /**
  * CoglMaterialFilter:
  * @COGL_MATERIAL_FILTER_NEAREST: Measuring in manhatten distance from the,
@@ -91,8 +96,10 @@ typedef enum {
  *   decide which of the above two to use. For cogl_rectangle(), it
  *   will use repeat mode if any of the texture coordinates are
  *   outside the range 0→1, otherwise it will use clamp to edge. For
- *   cogl_polygon() and cogl_vertex_buffer_draw() it will always use
- *   repeat mode. This is the default value.
+ *   cogl_polygon() it will always use repeat mode. For
+ *   cogl_vertex_buffer_draw() it will use repeat mode except for
+ *   layers that have point sprite coordinate generation enabled. This
+ *   is the default value.
  *
  * The wrap mode specifies what happens when texture coordinates
  * outside the range 0→1 are used. Note that if the filter mode is
@@ -122,16 +129,16 @@ typedef enum {
  *
  * Allocates and initializes a blank white material
  *
- * Return value: a handle to the new material
+ * Return value: a pointer to a new #CoglMaterial
  */
-CoglHandle
+CoglMaterial *
 cogl_material_new (void);
 
 /**
  * cogl_material_copy:
- * @source: the handle for the material to copy
+ * @source: a #CoglMaterial object to copy
  *
- * Creates a new material handle with the configuration copied from the
+ * Creates a new material with the configuration copied from the
  * source material.
  *
  * We would strongly advise developers to always aim to use
@@ -140,39 +147,39 @@ cogl_material_new (void);
  * keep track of a materials ancestry which we may use to help minimize GPU
  * state changes.
  *
- * Returns: a handle to the new material
+ * Returns: a pointer to the newly allocated #CoglMaterial
  *
  * Since: 1.2
  */
-CoglHandle
-cogl_material_copy (CoglHandle source);
+CoglMaterial *
+cogl_material_copy (CoglMaterial *source);
 
 #ifndef COGL_DISABLE_DEPRECATED
 
 /**
  * cogl_material_ref:
- * @handle: a @CoglHandle.
+ * @material: a #CoglMaterial object.
  *
- * Increment the reference count for a cogl material.
+ * Increment the reference count for a #CoglMaterial.
  *
- * Return value: the @handle.
+ * Return value: the @material.
  *
  * Since: 1.0
  *
- * Deprecated: 1.2: Use cogl_handle_ref() instead
+ * Deprecated: 1.2: Use cogl_object_ref() instead
  */
 CoglHandle
 cogl_material_ref (CoglHandle handle) G_GNUC_DEPRECATED;
 
 /**
  * cogl_material_unref:
- * @handle: a @CoglHandle.
+ * @material: a #CoglMaterial object.
  *
- * Decrement the reference count for a cogl material.
+ * Decrement the reference count for a #CoglMaterial.
  *
  * Since: 1.0
  *
- * Deprecated: 1.2: Use cogl_handle_unref() instead
+ * Deprecated: 1.2: Use cogl_object_unref() instead
  */
 void
 cogl_material_unref (CoglHandle handle) G_GNUC_DEPRECATED;
@@ -193,7 +200,7 @@ cogl_is_material (CoglHandle handle);
 
 /**
  * cogl_material_set_color:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @color: The components of the color
  *
  * Sets the basic color of the material, used when no lighting is enabled.
@@ -208,12 +215,12 @@ cogl_is_material (CoglHandle handle);
  * Since: 1.0
  */
 void
-cogl_material_set_color (CoglHandle       material,
+cogl_material_set_color (CoglMaterial    *material,
                          const CoglColor *color);
 
 /**
  * cogl_material_set_color4ub:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @red: The red component
  * @green: The green component
  * @blue: The blue component
@@ -226,15 +233,15 @@ cogl_material_set_color (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_set_color4ub (CoglHandle material,
-			    guint8     red,
-                            guint8     green,
-                            guint8     blue,
-                            guint8     alpha);
+cogl_material_set_color4ub (CoglMaterial *material,
+			    guint8        red,
+                            guint8        green,
+                            guint8        blue,
+                            guint8        alpha);
 
 /**
  * cogl_material_set_color4f:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @red: The red component
  * @green: The green component
  * @blue: The blue component
@@ -247,15 +254,15 @@ cogl_material_set_color4ub (CoglHandle material,
  * Since: 1.0
  */
 void
-cogl_material_set_color4f (CoglHandle material,
-                           float      red,
-                           float      green,
-                           float      blue,
-                           float      alpha);
+cogl_material_set_color4f (CoglMaterial *material,
+                           float         red,
+                           float         green,
+                           float         blue,
+                           float         alpha);
 
 /**
  * cogl_material_get_color:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @color: (out): The location to store the color
  *
  * Retrieves the current material color.
@@ -263,12 +270,12 @@ cogl_material_set_color4f (CoglHandle material,
  * Since: 1.0
  */
 void
-cogl_material_get_color (CoglHandle  material,
-                         CoglColor  *color);
+cogl_material_get_color (CoglMaterial *material,
+                         CoglColor    *color);
 
 /**
  * cogl_material_set_ambient:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @ambient: The components of the desired ambient color
  *
  * Sets the material's ambient color, in the standard OpenGL lighting
@@ -283,12 +290,12 @@ cogl_material_get_color (CoglHandle  material,
  * Since: 1.0
  */
 void
-cogl_material_set_ambient (CoglHandle       material,
+cogl_material_set_ambient (CoglMaterial    *material,
 			   const CoglColor *ambient);
 
 /**
  * cogl_material_get_ambient:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @ambient: The location to store the ambient color
  *
  * Retrieves the current ambient color for @material
@@ -296,12 +303,12 @@ cogl_material_set_ambient (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_get_ambient (CoglHandle  material,
-                           CoglColor  *ambient);
+cogl_material_get_ambient (CoglMaterial *material,
+                           CoglColor    *ambient);
 
 /**
  * cogl_material_set_diffuse:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @diffuse: The components of the desired diffuse color
  *
  * Sets the material's diffuse color, in the standard OpenGL lighting
@@ -313,12 +320,12 @@ cogl_material_get_ambient (CoglHandle  material,
  * Since: 1.0
  */
 void
-cogl_material_set_diffuse (CoglHandle       material,
+cogl_material_set_diffuse (CoglMaterial    *material,
 			   const CoglColor *diffuse);
 
 /**
  * cogl_material_get_diffuse:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @diffuse: The location to store the diffuse color
  *
  * Retrieves the current diffuse color for @material
@@ -326,12 +333,12 @@ cogl_material_set_diffuse (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_get_diffuse (CoglHandle  material,
-                           CoglColor  *diffuse);
+cogl_material_get_diffuse (CoglMaterial *material,
+                           CoglColor    *diffuse);
 
 /**
  * cogl_material_set_ambient_and_diffuse:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @color: The components of the desired ambient and diffuse colors
  *
  * Conveniently sets the diffuse and ambient color of @material at the same
@@ -344,12 +351,12 @@ cogl_material_get_diffuse (CoglHandle  material,
  * Since: 1.0
  */
 void
-cogl_material_set_ambient_and_diffuse (CoglHandle       material,
+cogl_material_set_ambient_and_diffuse (CoglMaterial    *material,
 				       const CoglColor *color);
 
 /**
  * cogl_material_set_specular:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @specular: The components of the desired specular color
  *
  * Sets the material's specular color, in the standard OpenGL lighting
@@ -361,12 +368,12 @@ cogl_material_set_ambient_and_diffuse (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_set_specular (CoglHandle       material,
+cogl_material_set_specular (CoglMaterial    *material,
 			    const CoglColor *specular);
 
 /**
  * cogl_material_get_specular:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @specular: The location to store the specular color
  *
  * Retrieves the materials current specular color.
@@ -374,29 +381,30 @@ cogl_material_set_specular (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_get_specular (CoglHandle  material,
-                            CoglColor  *specular);
+cogl_material_get_specular (CoglMaterial *material,
+                            CoglColor    *specular);
 
 /**
  * cogl_material_set_shininess:
- * @material: A CoglMaterial object
- * @shininess: The desired shininess; range: [0.0, 1.0]
+ * @material: A #CoglMaterial object
+ * @shininess: The desired shininess; must be >= 0.0
  *
- * Sets the materials shininess, in the standard OpenGL lighting model,
- * which determines how specular highlights are calculated. A higher
- * @shininess will produce smaller brigher highlights.
+ * Sets the shininess of the material, in the standard OpenGL lighting
+ * model, which determines the size of the specular highlights. A
+ * higher @shininess will produce smaller highlights which makes the
+ * object appear more shiny.
  *
  * The default value is 0.0
  *
  * Since: 1.0
  */
 void
-cogl_material_set_shininess (CoglHandle material,
-			     float      shininess);
+cogl_material_set_shininess (CoglMaterial *material,
+			     float         shininess);
 
 /**
  * cogl_material_get_shininess:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  *
  * Retrieves the materials current emission color.
  *
@@ -405,11 +413,11 @@ cogl_material_set_shininess (CoglHandle material,
  * Since: 1.0
  */
 float
-cogl_material_get_shininess (CoglHandle material);
+cogl_material_get_shininess (CoglMaterial *material);
 
 /**
  * cogl_material_set_emission:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @emission: The components of the desired emissive color
  *
  * Sets the material's emissive color, in the standard OpenGL lighting
@@ -421,12 +429,12 @@ cogl_material_get_shininess (CoglHandle material);
  * Since: 1.0
  */
 void
-cogl_material_set_emission (CoglHandle       material,
+cogl_material_set_emission (CoglMaterial    *material,
 			    const CoglColor *emission);
 
 /**
  * cogl_material_get_emission:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @emission: The location to store the emission color
  *
  * Retrieves the materials current emission color.
@@ -434,8 +442,8 @@ cogl_material_set_emission (CoglHandle       material,
  * Since: 1.0
  */
 void
-cogl_material_get_emission (CoglHandle material,
-                            CoglColor *emission);
+cogl_material_get_emission (CoglMaterial *material,
+                            CoglColor    *emission);
 
 /**
  * CoglMaterialAlphaFunc:
@@ -472,7 +480,7 @@ typedef enum {
 
 /**
  * cogl_material_set_alpha_test_function:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @alpha_func: A @CoglMaterialAlphaFunc constant
  * @alpha_reference: A reference point that the chosen alpha function uses
  *   to compare incoming fragments to.
@@ -488,13 +496,13 @@ typedef enum {
  * Since: 1.0
  */
 void
-cogl_material_set_alpha_test_function (CoglHandle            material,
+cogl_material_set_alpha_test_function (CoglMaterial         *material,
 				       CoglMaterialAlphaFunc alpha_func,
 				       float                 alpha_reference);
 
 /**
  * cogl_material_set_blend:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @blend_string: A <link linkend="cogl-Blend-Strings">Cogl blend string</link>
  *   describing the desired blend function.
  * @error: return location for a #GError that may report lack of driver
@@ -577,13 +585,13 @@ cogl_material_set_alpha_test_function (CoglHandle            material,
  * Since: 1.0
  */
 gboolean
-cogl_material_set_blend (CoglHandle  material,
-                         const char *blend_string,
-                         GError    **error);
+cogl_material_set_blend (CoglMaterial *material,
+                         const char   *blend_string,
+                         GError      **error);
 
 /**
  * cogl_material_set_blend_constant:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @constant_color: The constant color you want
  *
  * When blending is setup to reference a CONSTANT blend factor then
@@ -592,12 +600,45 @@ cogl_material_set_blend (CoglHandle  material,
  * Since: 1.0
  */
 void
-cogl_material_set_blend_constant (CoglHandle material,
+cogl_material_set_blend_constant (CoglMaterial *material,
                                   const CoglColor *constant_color);
 
+/**
+ * cogl_material_set_point_size:
+ * @material: a #CoglHandle to a material.
+ * @size: the new point size.
+ *
+ * Changes the size of points drawn when %COGL_VERTICES_MODE_POINTS is
+ * used with the vertex buffer API. Note that typically the GPU will
+ * only support a limited minimum and maximum range of point sizes. If
+ * the chosen point size is outside that range then the nearest value
+ * within that range will be used instead. The size of a point is in
+ * screen space so it will be the same regardless of any
+ * transformations. The default point size is 1.0.
+ *
+ * Since: 1.4
+ */
+void
+cogl_material_set_point_size (CoglHandle material,
+                              float      point_size);
+
+/**
+ * cogl_material_get_point_size:
+ * @material: a #CoglHandle to a material.
+ *
+ * Get the size of points drawn when %COGL_VERTICES_MODE_POINTS is
+ * used with the vertex buffer API.
+ *
+ * Return value: the point size of the material.
+ *
+ * Since: 1.4
+ */
+float
+cogl_material_get_point_size (CoglHandle  material);
+
 /**
  * cogl_material_set_layer:
- * @material: A #CoglHandle for a material object
+ * @material: A #CoglMaterial object
  * @layer_index: the index of the layer
  * @texture: a #CoglHandle for the layer object
  *
@@ -615,25 +656,25 @@ cogl_material_set_blend_constant (CoglHandle material,
  * Since: 1.0
  */
 void
-cogl_material_set_layer (CoglHandle material,
-			 int        layer_index,
-			 CoglHandle texture);
+cogl_material_set_layer (CoglMaterial *material,
+			 int           layer_index,
+			 CoglHandle    texture);
 
 /**
  * cogl_material_remove_layer:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @layer_index: Specifies the layer you want to remove
  *
  * This function removes a layer from your material
  */
 void
-cogl_material_remove_layer (CoglHandle material,
-			    int        layer_index);
+cogl_material_remove_layer (CoglMaterial *material,
+			    int           layer_index);
 
 
 /**
  * cogl_material_set_layer_combine:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @layer_index: Specifies the layer you want define a combine function for
  * @blend_string: A <link linkend="cogl-Blend-Strings">Cogl blend string</link>
  *    describing the desired texture combine function.
@@ -724,14 +765,14 @@ cogl_material_remove_layer (CoglHandle material,
  * Since: 1.0
  */
 gboolean
-cogl_material_set_layer_combine (CoglHandle   material,
-				 int          layer_index,
-				 const char  *blend_string,
-                                 GError     **error);
+cogl_material_set_layer_combine (CoglMaterial *material,
+				 int           layer_index,
+				 const char   *blend_string,
+                                 GError      **error);
 
 /**
  * cogl_material_set_layer_combine_constant:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @layer_index: Specifies the layer you want to specify a constant used
  *               for texture combining
  * @constant: The constant color you want
@@ -742,13 +783,13 @@ cogl_material_set_layer_combine (CoglHandle   material,
  * Since: 1.0
  */
 void
-cogl_material_set_layer_combine_constant (CoglHandle       material,
+cogl_material_set_layer_combine_constant (CoglMaterial    *material,
                                           int              layer_index,
                                           const CoglColor *constant);
 
 /**
  * cogl_material_set_layer_matrix:
- * @material: A CoglMaterial object
+ * @material: A #CoglMaterial object
  * @layer_index: the index for the layer inside @material
  * @matrix: the transformation matrix for the layer
  *
@@ -756,28 +797,37 @@ cogl_material_set_layer_combine_constant (CoglHandle       material,
  * and rotate a single layer of a material used to fill your geometry.
  */
 void
-cogl_material_set_layer_matrix (CoglHandle        material,
-                                int               layer_index,
-                                const CoglMatrix *matrix);
+cogl_material_set_layer_matrix (CoglMaterial     *material,
+				int               layer_index,
+				const CoglMatrix *matrix);
 
 /**
  * cogl_material_get_layers:
- * @material: a #CoglHandle for a material
+ * @material: A #CoglMaterial object
  *
- * This function lets you access a materials internal list of layers
+ * This function lets you access a material's internal list of layers
  * for iteration.
  *
- * Return value: (element-type Handle) (transfer none): A list of
- *   #CoglHandle<!-- -->'s that can be passed to the  cogl_material_layer_*
- *   functions. The list is owned by COGL and it  should not be modified or
- *   freed
+ * <note>You should avoid using this API if possible since it was only
+ * made public by mistake and will be deprecated when we have
+ * suitable alternative.</note>
+ *
+ * <note>It's important to understand that the list returned may not
+ * remain valid if you modify the material or any of the layers in any
+ * way and so you would have to re-get the list in that
+ * situation.</note>
+ *
+ * Return value: (element-type CoglMaterialLayer) (transfer none): A
+ *    list of #CoglMaterialLayer<!-- -->'s that can be passed to the
+ *    cogl_material_layer_* functions. The list is owned by Cogl and it
+ *    should not be modified or freed
  */
 G_CONST_RETURN GList *
-cogl_material_get_layers (CoglHandle material);
+cogl_material_get_layers (CoglMaterial *material);
 
 /**
  * cogl_material_get_n_layers:
- * @material: a #CoglHandle for a material
+ * @material: A #CoglMaterial object
  *
  * Retrieves the number of layers defined for the given @material
  *
@@ -786,7 +836,7 @@ cogl_material_get_layers (CoglHandle material);
  * Since: 1.0
  */
 int
-cogl_material_get_n_layers (CoglHandle material);
+cogl_material_get_n_layers (CoglMaterial *material);
 
 /**
  * CoglMaterialLayerType:
@@ -805,7 +855,7 @@ typedef enum {
 
 /**
  * cogl_material_layer_get_type:
- * @layer: A #CoglHandle for a material layer
+ * @layer: A #CoglMaterialLayer object
  *
  * Retrieves the type of the layer
  *
@@ -817,11 +867,11 @@ typedef enum {
  * Return value: the type of the layer
  */
 CoglMaterialLayerType
-cogl_material_layer_get_type (CoglHandle layer);
+cogl_material_layer_get_type (CoglMaterialLayer *layer);
 
 /**
  * cogl_material_layer_get_texture:
- * @layer: A #CoglHandle for a material layer
+ * @layer: A #CoglMaterialLayer object
  *
  * Extracts a texture handle for a specific layer.
  *
@@ -834,7 +884,7 @@ cogl_material_layer_get_type (CoglHandle layer);
  * Return value: a #CoglHandle for the texture inside the layer
  */
 CoglHandle
-cogl_material_layer_get_texture (CoglHandle layer);
+cogl_material_layer_get_texture (CoglMaterialLayer *layer);
 
 /**
  * cogl_material_layer_get_min_filter:
@@ -845,22 +895,22 @@ cogl_material_layer_get_texture (CoglHandle layer);
  * Return value: the current downscaling filter
  */
 CoglMaterialFilter
-cogl_material_layer_get_min_filter (CoglHandle layer);
+cogl_material_layer_get_min_filter (CoglMaterialLayer *layer);
 
 /**
  * cogl_material_layer_get_mag_filter:
- * @layer: a #CoglHandle for a material layer
+ * @layer: A #CoglMaterialLayer object
  *
  * Queries the currently set downscaling filter for a material later
  *
  * Return value: the current downscaling filter
  */
 CoglMaterialFilter
-cogl_material_layer_get_mag_filter (CoglHandle layer);
+cogl_material_layer_get_mag_filter (CoglMaterialLayer *layer);
 
 /**
  * cogl_material_set_layer_filters:
- * @material: a #CoglHandle to a material.
+ * @material: A #CoglMaterial object
  * @layer_index: the layer number to change.
  * @min_filter: the filter used when scaling a texture down.
  * @mag_filter: the filter used when magnifying a texture.
@@ -869,15 +919,59 @@ cogl_material_layer_get_mag_filter (CoglHandle layer);
  * drawn at other scales than 100%.
  */
 void
-cogl_material_set_layer_filters (CoglHandle         material,
+cogl_material_set_layer_filters (CoglMaterial      *material,
                                  int                layer_index,
                                  CoglMaterialFilter min_filter,
                                  CoglMaterialFilter mag_filter);
 
 /**
- * cogl_material_set_layer_wrap_mode_s:
+ * cogl_material_set_layer_point_sprite_coords_enabled:
  * @material: a #CoglHandle to a material.
  * @layer_index: the layer number to change.
+ * @enable: whether to enable point sprite coord generation.
+ * @error: A return location for a GError, or NULL to ignore errors.
+ *
+ * When rendering points, if @enable is %TRUE then the texture
+ * coordinates for this layer will be replaced with coordinates that
+ * vary from 0.0 to 1.0 across the primitive. The top left of the
+ * point will have the coordinates 0.0,0.0 and the bottom right will
+ * have 1.0,1.0. If @enable is %FALSE then the coordinates will be
+ * fixed for the entire point.
+ *
+ * This function will only work if %COGL_FEATURE_POINT_SPRITE is
+ * available. If the feature is not available then the function will
+ * return %FALSE and set @error.
+ *
+ * Return value: %TRUE if the function succeeds, %FALSE otherwise.
+ * Since: 1.4
+ */
+gboolean
+cogl_material_set_layer_point_sprite_coords_enabled (CoglMaterial *material,
+                                                     int           layer_index,
+                                                     gboolean      enable,
+                                                     GError      **error);
+
+/**
+ * cogl_material_get_layer_point_sprite_coords_enabled:
+ * @material: a #CoglHandle to a material.
+ * @layer_index: the layer number to check.
+ *
+ * Gets whether point sprite coordinate generation is enabled for this
+ * texture layer.
+ *
+ * Return value: whether the texture coordinates will be replaced with
+ * point sprite coordinates.
+ *
+ * Since: 1.4
+ */
+gboolean
+cogl_material_get_layer_point_sprite_coords_enabled (CoglMaterial *material,
+                                                     int           layer_index);
+
+/**
+ * cogl_material_set_layer_wrap_mode_s:
+ * @material: A #CoglMaterial object
+ * @layer_index: the layer number to change.
  * @mode: the new wrap mode
  *
  * Sets the wrap mode for the 's' coordinate of texture lookups on this layer.
@@ -885,13 +979,13 @@ cogl_material_set_layer_filters (CoglHandle         material,
  * Since: 1.4
  */
 void
-cogl_material_set_layer_wrap_mode_s (CoglHandle           material,
+cogl_material_set_layer_wrap_mode_s (CoglMaterial        *material,
                                      int                  layer_index,
                                      CoglMaterialWrapMode mode);
 
 /**
  * cogl_material_set_layer_wrap_mode_t:
- * @material: a #CoglHandle to a material.
+ * @material: A #CoglMaterial object
  * @layer_index: the layer number to change.
  * @mode: the new wrap mode
  *
@@ -900,50 +994,84 @@ cogl_material_set_layer_wrap_mode_s (CoglHandle           material,
  * Since: 1.4
  */
 void
-cogl_material_set_layer_wrap_mode_t (CoglHandle           material,
+cogl_material_set_layer_wrap_mode_t (CoglMaterial        *material,
+                                     int                  layer_index,
+                                     CoglMaterialWrapMode mode);
+
+/**
+ * cogl_material_set_layer_wrap_mode_p:
+ * @material: A #CoglMaterial object
+ * @layer_index: the layer number to change.
+ * @mode: the new wrap mode
+ *
+ * Sets the wrap mode for the 'p' coordinate of texture lookups on
+ * this layer. 'p' is the third coordinate.
+ *
+ * Since: 1.4
+ */
+void
+cogl_material_set_layer_wrap_mode_p (CoglMaterial        *material,
                                      int                  layer_index,
                                      CoglMaterialWrapMode mode);
 
 /**
  * cogl_material_set_layer_wrap_mode:
- * @material: a #CoglHandle to a material.
+ * @material: A #CoglMaterial object
  * @layer_index: the layer number to change.
  * @mode: the new wrap mode
  *
- * Sets the wrap mode for both coordinates of texture lookups on this
- * layer. This is equivalent to calling
- * cogl_material_set_layer_wrap_mode_s() and
- * cogl_material_set_layer_wrap_mode_t() separately.
+ * Sets the wrap mode for all three coordinates of texture lookups on
+ * this layer. This is equivalent to calling
+ * cogl_material_set_layer_wrap_mode_s(),
+ * cogl_material_set_layer_wrap_mode_t() and
+ * cogl_material_set_layer_wrap_mode_p() separately.
  *
  * Since: 1.4
  */
 void
-cogl_material_set_layer_wrap_mode (CoglHandle           material,
+cogl_material_set_layer_wrap_mode (CoglMaterial        *material,
                                    int                  layer_index,
                                    CoglMaterialWrapMode mode);
 
 /**
  * cogl_material_layer_get_wrap_mode_s:
- * @layer: a #CoglHandle to a material mayer.
+ * @layer: A #CoglMaterialLayer object
  *
  * Gets the wrap mode for the 's' coordinate of texture lookups on this layer.
  *
+ * Return value: the wrap mode value for the s coordinate.
+ *
  * Since: 1.4
  */
 CoglMaterialWrapMode
-cogl_material_layer_get_wrap_mode_s (CoglHandle layer);
+cogl_material_layer_get_wrap_mode_s (CoglMaterialLayer *layer);
 
 /**
  * cogl_material_layer_get_wrap_mode_t:
- * @layer: a #CoglHandle to a material mayer.
+ * @layer: A #CoglMaterialLayer object
  *
  * Gets the wrap mode for the 't' coordinate of texture lookups on this layer.
  *
+ * Return value: the wrap mode value for the t coordinate.
+ *
  * Since: 1.4
  */
 CoglMaterialWrapMode
-cogl_material_layer_get_wrap_mode_t (CoglHandle layer);
+cogl_material_layer_get_wrap_mode_t (CoglMaterialLayer *layer);
 
+/**
+ * cogl_material_layer_get_wrap_mode_p:
+ * @layer: A #CoglMaterialLayer object
+ *
+ * Gets the wrap mode for the 'p' coordinate of texture lookups on
+ * this layer. 'p' is the third coordinate.
+ *
+ * Return value: the wrap mode value for the p coordinate.
+ *
+ * Since: 1.4
+ */
+CoglMaterialWrapMode
+cogl_material_layer_get_wrap_mode_p (CoglMaterialLayer *layer);
 
 /* XXX: should this be CoglMaterialDepthTestFunction?
  * It makes it very verbose but would be consistent with
@@ -995,7 +1123,7 @@ typedef enum
 
 /**
  * cogl_material_set_depth_test_enabled:
- * @handle: A CoglMaterial handle
+ * @material: A #CoglMaterial object
  * @enable: The enable state you want
  *
  * Enables or disables depth testing according to the value of
@@ -1012,27 +1140,29 @@ typedef enum
  * By default depth testing is disabled.
  *
  * Since: 1.4
+ * Stability: Unstable
  */
 void
-cogl_material_set_depth_test_enabled (CoglHandle handle,
+cogl_material_set_depth_test_enabled (CoglMaterial *material,
                                       gboolean enable);
 
 /**
  * cogl_material_get_depth_test_enabled:
- * @handle: A CoglMaterial handle
+ * @material: A #CoglMaterial object
  *
  * Gets the current depth test enabled state as previously set by
  * cogl_material_set_depth_test_enabled().
  *
  * Returns: The material's current depth test enabled state.
  * Since: 1.4
+ * Stability: Unstable
  */
 gboolean
-cogl_material_get_depth_test_enabled (CoglHandle handle);
+cogl_material_get_depth_test_enabled (CoglMaterial *material);
 
 /**
  * cogl_material_set_depth_writing_enabled:
- * @handle: A CoglMaterial handle
+ * @material: A #CoglMaterial object
  * @enable: The enable state you want
  *
  * Enables or disables depth buffer writing according to the value of
@@ -1044,28 +1174,29 @@ cogl_material_get_depth_test_enabled (CoglHandle handle);
  * By default depth writing is enabled
  *
  * Since: 1.4
+ * Stability: Unstable
  */
 void
-cogl_material_set_depth_writing_enabled (CoglHandle handle,
+cogl_material_set_depth_writing_enabled (CoglMaterial *material,
                                          gboolean enable);
 
 /**
  * cogl_material_get_depth_writing_enabled:
- * @handle: A CoglMaterial handle
- * Since: 1.4
+ * @material: A #CoglMaterial object
  *
  * Gets the depth writing enable state as set by the corresponding
  * cogl_material_set_depth_writing_enabled.
  *
  * Returns: The current depth writing enable state
  * Since: 1.4
+ * Stability: Unstable
  */
 gboolean
-cogl_material_get_depth_writing_enabled (CoglHandle handle);
+cogl_material_get_depth_writing_enabled (CoglMaterial *material);
 
 /**
  * cogl_material_set_depth_test_function:
- * @handle: A CoglMaterial handle
+ * @material: A #CoglMaterial object
  * @function: The #CoglDepthTestFunction to set
  *
  * Sets the #CoglDepthTestFunction used to compare the depth value of
@@ -1073,27 +1204,29 @@ cogl_material_get_depth_writing_enabled (CoglHandle handle);
  * depth buffer.
  *
  * Since: 1.4
+ * Stability: Unstable
  */
 void
-cogl_material_set_depth_test_function (CoglHandle handle,
+cogl_material_set_depth_test_function (CoglMaterial *material,
                                        CoglDepthTestFunction function);
 
 /**
  * cogl_material_get_depth_test_function:
- * @handle: A CoglMaterial handle
+ * @material: A #CoglMaterial object
  *
  * Gets the current depth test enable state as previously set via
  * cogl_material_set_depth_test_enabled().
  *
  * Returns: The current depth test enable state.
  * Since: 1.4
+ * Stability: Unstable
  */
 CoglDepthTestFunction
-cogl_material_get_depth_test_function (CoglHandle handle);
+cogl_material_get_depth_test_function (CoglMaterial *material);
 
 /**
  * cogl_material_set_depth_range:
- * @handle: A CoglMaterial handle
+ * @material: A #CoglMaterial object
  * @near_val: The near component of the desired depth range which will be
  * clamped to the range [0, 1]
  * @far_val: The far component of the desired depth range which will be
@@ -1125,16 +1258,17 @@ cogl_material_get_depth_test_function (CoglHandle handle);
  * Returns: %TRUE if driver support is available else %FALSE.
  *
  * Since: 1.4
+ * Stability: Unstable
  */
 gboolean
-cogl_material_set_depth_range (CoglHandle handle,
+cogl_material_set_depth_range (CoglMaterial *material,
                                float near_val,
                                float far_val,
                                GError **error);
 
 /**
  * cogl_material_get_depth_range_mapping:
- * @handle: A CoglMaterial handle
+ * @material: A #CoglMaterial object
  * @near_val: A pointer to store the near component of the depth range
  * @far_val: A pointer to store the far component of the depth range
  *
@@ -1143,9 +1277,10 @@ cogl_material_set_depth_range (CoglHandle handle,
  * set with cogl_material_set_depth_range().
  *
  * Since: 1.4
+ * Stability: Unstable
  */
 void
-cogl_material_get_depth_range (CoglHandle handle,
+cogl_material_get_depth_range (CoglMaterial *material,
                                float *near_val,
                                float *far_val);
 
