@@ -137,6 +137,7 @@
 #include "cogl-vertex-buffer-private.h"
 #include "cogl-texture-private.h"
 #include "cogl-material-private.h"
+#include "cogl-material-opengl-private.h"
 #include "cogl-primitives.h"
 #include "cogl-framebuffer-private.h"
 #include "cogl-journal-private.h"
@@ -183,7 +184,7 @@
 
 #define glVertexAttribPointer ctx->drv.pf_glVertexAttribPointer
 #define glEnableVertexAttribArray ctx->drv.pf_glEnableVertexAttribArray
-#define glDisableVertexAttribArray ctx->drv.pf_glEnableVertexAttribArray
+#define glDisableVertexAttribArray ctx->drv.pf_glDisableVertexAttribArray
 #define MAY_HAVE_PROGRAMABLE_GL
 
 #elif defined (HAVE_COGL_GLES2)
@@ -212,6 +213,7 @@ static void _cogl_vertex_buffer_free (CoglVertexBuffer *buffer);
 static void _cogl_vertex_buffer_indices_free (CoglVertexBufferIndices *buffer_indices);
 
 COGL_HANDLE_DEFINE (VertexBuffer, vertex_buffer);
+COGL_OBJECT_DEFINE_DEPRECATED_REF_COUNTING (vertex_buffer);
 COGL_HANDLE_DEFINE (VertexBufferIndices, vertex_buffer_indices);
 
 CoglHandle
@@ -1659,6 +1661,37 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
       if (tex_handle == COGL_INVALID_HANDLE)
         continue;
 
+      if (!cogl_material_get_layer_point_sprite_coords_enabled (source, i))
+        {
+          /* By default COGL_MATERIAL_WRAP_MODE_AUTOMATIC becomes
+             GL_CLAMP_TO_EDGE but we want GL_REPEAT to maintain
+             compatibility with older versions of Cogl so we'll
+             override it. We don't want to do this for point sprites
+             because in that case the whole texture is drawn so you
+             would usually want clamp-to-edge. */
+          if (cogl_material_layer_get_wrap_mode_s (layer) ==
+              COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+            {
+              options.wrap_mode_overrides.values[i].s =
+                COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+              options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+            }
+          if (cogl_material_layer_get_wrap_mode_t (layer) ==
+              COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+            {
+              options.wrap_mode_overrides.values[i].t =
+                COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+              options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+            }
+          if (cogl_material_layer_get_wrap_mode_p (layer) ==
+              COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+            {
+              options.wrap_mode_overrides.values[i].p =
+                COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+              options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+            }
+        }
+
       /* Give the texture a chance to know that we're rendering
          non-quad shaped primitives. If the texture is in an atlas it
          will be migrated */
@@ -1691,32 +1724,6 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
            */
           fallback_layers |= (1 << i);
         }
-
-      /* By default COGL_MATERIAL_WRAP_MODE_AUTOMATIC becomes
-         GL_CLAMP_TO_EDGE but we want GL_REPEAT to maintain
-         compatibility with older versions of Cogl so we'll override
-         it */
-      if (cogl_material_layer_get_wrap_mode_s (layer) ==
-          COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
-        {
-          options.wrap_mode_overrides.values[i].s =
-            COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
-          options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
-        }
-      if (cogl_material_layer_get_wrap_mode_t (layer) ==
-          COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
-        {
-          options.wrap_mode_overrides.values[i].t =
-            COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
-          options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
-        }
-      if (_cogl_material_layer_get_wrap_mode_r (layer) ==
-          COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
-        {
-          options.wrap_mode_overrides.values[i].r =
-            COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
-          options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
-        }
     }
 
   /* Disable any tex coord arrays that we didn't use */
