@@ -37,6 +37,7 @@
 #include "cogl-util.h"
 #include "cogl-context.h"
 #include "cogl-material-private.h"
+#include "cogl-material-opengl-private.h"
 #include "cogl-winsys.h"
 #include "cogl-framebuffer-private.h"
 #include "cogl-matrix-private.h"
@@ -492,6 +493,9 @@ cogl_get_features (void)
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_VBOS))
     ctx->feature_flags &= ~COGL_FEATURE_VBOS;
 
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_PBOS))
+    ctx->feature_flags &= ~COGL_FEATURE_PBOS;
+
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_GLSL))
     ctx->feature_flags &= ~COGL_FEATURE_SHADERS_GLSL;
 
@@ -549,7 +553,7 @@ cogl_get_bitmasks (int *red,
                    int *blue,
                    int *alpha)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
 
   framebuffer = _cogl_get_framebuffer ();
 
@@ -573,48 +577,17 @@ cogl_set_fog (const CoglColor *fog_color,
               float            z_near,
               float            z_far)
 {
-  GLfloat fogColor[4];
-  GLenum gl_mode = GL_LINEAR;
-
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  /* The cogl journal doesn't currently track fog state changes */
-  _cogl_journal_flush ();
+  if (ctx->legacy_fog_state.enabled == FALSE)
+    ctx->legacy_state_set++;
 
-  fogColor[0] = cogl_color_get_red_float (fog_color);
-  fogColor[1] = cogl_color_get_green_float (fog_color);
-  fogColor[2] = cogl_color_get_blue_float (fog_color);
-  fogColor[3] = cogl_color_get_alpha_float (fog_color);
-
-  glEnable (GL_FOG);
-
-  glFogfv (GL_FOG_COLOR, fogColor);
-
-#if HAVE_COGL_GLES
-  switch (mode)
-    {
-    case COGL_FOG_MODE_LINEAR:
-      gl_mode = GL_LINEAR;
-      break;
-    case COGL_FOG_MODE_EXPONENTIAL:
-      gl_mode = GL_EXP;
-      break;
-    case COGL_FOG_MODE_EXPONENTIAL_SQUARED:
-      gl_mode = GL_EXP2;
-      break;
-    }
-#endif
-  /* TODO: support other modes for GLES2 */
-
-  /* NB: GLES doesn't have glFogi */
-  glFogf (GL_FOG_MODE, gl_mode);
-  glHint (GL_FOG_HINT, GL_NICEST);
-
-  glFogf (GL_FOG_DENSITY, (GLfloat) density);
-  glFogf (GL_FOG_START, (GLfloat) z_near);
-  glFogf (GL_FOG_END, (GLfloat) z_far);
-
-  ctx->fog_enabled = TRUE;
+  ctx->legacy_fog_state.enabled = TRUE;
+  ctx->legacy_fog_state.color = *fog_color;
+  ctx->legacy_fog_state.mode = mode;
+  ctx->legacy_fog_state.density = density;
+  ctx->legacy_fog_state.z_near = z_near;
+  ctx->legacy_fog_state.z_far = z_far;
 }
 
 void
@@ -622,11 +595,10 @@ cogl_disable_fog (void)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  /* Currently the journal can't track changes to fog state... */
-  _cogl_journal_flush ();
+  if (ctx->legacy_fog_state.enabled == TRUE)
+    ctx->legacy_state_set--;
 
-  glDisable (GL_FOG);
-  ctx->fog_enabled = FALSE;
+  ctx->legacy_fog_state.enabled = FALSE;
 }
 
 void
@@ -645,12 +617,14 @@ cogl_read_pixels (int x,
                   guint8 *pixels)
 {
   CoglFramebuffer *framebuffer;
-  int        framebuffer_height;
-  int        bpp;
-  CoglBitmap bmp;
-  GLenum     gl_intformat;
-  GLenum     gl_format;
-  GLenum     gl_type;
+  int              framebuffer_height;
+  int              bpp;
+  CoglBitmap      *bmp;
+  GLenum           gl_intformat;
+  GLenum           gl_format;
+  GLenum           gl_type;
+  CoglPixelFormat  bmp_format;
+  int              rowstride;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -677,11 +651,8 @@ cogl_read_pixels (int x,
 
   /* Initialise the CoglBitmap */
   bpp = _cogl_get_format_bpp (format);
-  bmp.format = format;
-  bmp.data = pixels;
-  bmp.width = width;
-  bmp.height = height;
-  bmp.rowstride = bpp * width;
+  rowstride = bpp * width;
+  bmp_format = format;
 
   if ((format & COGL_A_BIT))
     {
@@ -695,9 +666,13 @@ cogl_read_pixels (int x,
          it. Eventually we may want to add a way for an application to
          inform Cogl that the framebuffer is not premultiplied in case
          it is being used for some special purpose. */
-      bmp.format |= COGL_PREMULT_BIT;
+      bmp_format |= COGL_PREMULT_BIT;
     }
 
+  bmp = _cogl_bitmap_new_from_data (pixels,
+                                    bmp_format, width, height, rowstride,
+                                    NULL, NULL);
+
   _cogl_pixel_format_to_gl (format, &gl_intformat, &gl_format, &gl_type);
 
   /* Under GLES only GL_RGBA with GL_UNSIGNED_BYTE as well as an
@@ -711,33 +686,33 @@ cogl_read_pixels (int x,
 #ifndef COGL_HAS_GL
   if (gl_format != GL_RGBA || gl_type != GL_UNSIGNED_BYTE)
     {
-      CoglBitmap tmp_bmp, dst_bmp;
+      CoglBitmap *tmp_bmp, *dst_bmp;
+      guint8 *tmp_data = g_malloc (width * height * 4);
 
-      tmp_bmp.format = COGL_PIXEL_FORMAT_RGBA_8888_PRE;
-      tmp_bmp.data = g_malloc (width * height * 4);
-      tmp_bmp.width = width;
-      tmp_bmp.height = height;
-      tmp_bmp.rowstride = 4 * width;
+      tmp_bmp = _cogl_bitmap_new_from_data (tmp_data,
+                                            COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                            width, height, 4 * width,
+                                            (CoglBitmapDestroyNotify) g_free,
+                                            NULL);
 
-      _cogl_texture_driver_prep_gl_for_pixels_download (tmp_bmp.rowstride, 4);
+      _cogl_texture_driver_prep_gl_for_pixels_download (4 * width, 4);
 
       GE( glReadPixels (x, y, width, height,
                         GL_RGBA, GL_UNSIGNED_BYTE,
-                        tmp_bmp.data) );
+                        tmp_data) );
 
       /* CoglBitmap doesn't currently have a way to convert without
          allocating its own buffer so we have to copy the data
          again */
-      if (_cogl_bitmap_convert_format_and_premult (&tmp_bmp,
-                                                   &dst_bmp,
-                                                   bmp.format))
+      if ((dst_bmp = _cogl_bitmap_convert_format_and_premult (tmp_bmp,
+                                                              bmp_format)))
         {
-          _cogl_bitmap_copy_subregion (&dst_bmp,
-                                       &bmp,
+          _cogl_bitmap_copy_subregion (dst_bmp,
+                                       bmp,
                                        0, 0,
                                        0, 0,
-                                       bmp.width, bmp.height);
-          g_free (dst_bmp.data);
+                                       width, height);
+          cogl_object_unref (dst_bmp);
         }
       else
         {
@@ -745,26 +720,26 @@ cogl_read_pixels (int x,
              just have to leave the data initialised */
         }
 
-      g_free (tmp_bmp.data);
+      cogl_object_unref (tmp_bmp);
     }
   else
 #endif
     {
-      _cogl_texture_driver_prep_gl_for_pixels_download (bmp.rowstride, bpp);
+      _cogl_texture_driver_prep_gl_for_pixels_download (rowstride, bpp);
 
       GE( glReadPixels (x, y, width, height, gl_format, gl_type, pixels) );
 
       /* Convert to the premult format specified by the caller
          in-place. This will do nothing if the premult status is already
          correct. */
-      _cogl_bitmap_convert_premult_status (&bmp, format);
+      _cogl_bitmap_convert_premult_status (bmp, format);
     }
 
   /* NB: All offscreen rendering is done upside down so there is no need
    * to flip in this case... */
   if (!cogl_is_offscreen (framebuffer))
     {
-      guint8 *temprow = g_alloca (bmp.rowstride * sizeof (guint8));
+      guint8 *temprow = g_alloca (rowstride * sizeof (guint8));
 
       /* TODO: consider using the GL_MESA_pack_invert extension in the future
        * to avoid this flip... */
@@ -775,15 +750,17 @@ cogl_read_pixels (int x,
           if (y != height - y - 1) /* skip center row */
             {
               memcpy (temprow,
-                      pixels + y * bmp.rowstride, bmp.rowstride);
-              memcpy (pixels + y * bmp.rowstride,
-                      pixels + (height - y - 1) * bmp.rowstride, bmp.rowstride);
-              memcpy (pixels + (height - y - 1) * bmp.rowstride,
+                      pixels + y * rowstride, rowstride);
+              memcpy (pixels + y * rowstride,
+                      pixels + (height - y - 1) * rowstride, rowstride);
+              memcpy (pixels + (height - y - 1) * rowstride,
                       temprow,
-                      bmp.rowstride);
+                      rowstride);
             }
         }
     }
+
+  cogl_object_unref (bmp);
 }
 
 static void
