@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 /* This is included multiple times with different definitions for
@@ -30,7 +30,8 @@ COGL_FEATURE_BEGIN (offscreen, 255, 255,
                        functions */
                     "ARB:\0EXT\0",
                     "framebuffer_object\0",
-                    COGL_FEATURE_OFFSCREEN)
+                    COGL_FEATURE_OFFSCREEN,
+                    0)
 COGL_FEATURE_FUNCTION (void, glGenRenderbuffers,
                        (GLsizei               n,
                         GLuint               *renderbuffers))
@@ -69,12 +70,18 @@ COGL_FEATURE_FUNCTION (void, glDeleteFramebuffers,
                         const                 GLuint *framebuffers))
 COGL_FEATURE_FUNCTION (void, glGenerateMipmap,
                        (GLenum                target))
+COGL_FEATURE_FUNCTION (void, glGetFramebufferAttachmentParameteriv,
+                       (GLenum                target,
+                        GLenum                attachment,
+                        GLenum                pname,
+                        GLint                *params))
 COGL_FEATURE_END ()
 
 COGL_FEATURE_BEGIN (offscreen_blit, 255, 255,
                     "EXT\0",
                     "framebuffer_blit\0",
-                    COGL_FEATURE_OFFSCREEN_BLIT)
+                    COGL_FEATURE_OFFSCREEN_BLIT,
+                    0)
 COGL_FEATURE_FUNCTION (void, glBlitFramebuffer,
                        (GLint                 srcX0,
                         GLint                 srcY0,
@@ -91,7 +98,8 @@ COGL_FEATURE_END ()
 COGL_FEATURE_BEGIN (offscreen_multisample, 255, 255,
                     "EXT\0",
                     "framebuffer_multisample\0",
-                    COGL_FEATURE_OFFSCREEN_MULTISAMPLE)
+                    COGL_FEATURE_OFFSCREEN_MULTISAMPLE,
+                    0)
 COGL_FEATURE_FUNCTION (void, glRenderbufferStorageMultisample,
                        (GLenum                target,
                         GLsizei               samples,
@@ -99,50 +107,79 @@ COGL_FEATURE_FUNCTION (void, glRenderbufferStorageMultisample,
                         GLsizei               width,
                         GLsizei               height))
 COGL_FEATURE_END ()
+
 COGL_FEATURE_BEGIN (read_pixels_async, 2, 1,
                     "EXT\0",
                     "pixel_buffer_object\0",
-                    COGL_FEATURE_PBOS)
+                    COGL_FEATURE_PBOS,
+                    0)
+COGL_FEATURE_END ()
+
+/* ARB_fragment_program */
+COGL_FEATURE_BEGIN (arbfp, 255, 255,
+                    "ARB\0",
+                    "fragment_program\0",
+                    0,
+                    COGL_FEATURE_PRIVATE_ARB_FP)
+COGL_FEATURE_FUNCTION (void, glGenPrograms,
+                       (GLsizei               n,
+                        GLuint               *programs))
+COGL_FEATURE_FUNCTION (void, glDeletePrograms,
+                       (GLsizei               n,
+                        GLuint               *programs))
+COGL_FEATURE_FUNCTION (void, glBindProgram,
+                       (GLenum                target,
+                        GLuint                program))
+COGL_FEATURE_FUNCTION (void, glProgramString,
+                       (GLenum                target,
+                        GLenum                format,
+                        GLsizei               len,
+                        const void           *program))
+COGL_FEATURE_FUNCTION (void, glProgramLocalParameter4fv,
+                       (GLenum                target,
+                        GLuint                index,
+                        GLfloat              *params))
 COGL_FEATURE_END ()
 
 /* The function names in OpenGL 2.0 are different so we can't easily
    just check for GL 2.0 */
-COGL_FEATURE_BEGIN (shaders_glsl, 255, 255,
-                    "ARB\0",
-                    "shader_objects\0"
-                    "vertex_shader\0"
-                    "fragment_shader\0",
-                    COGL_FEATURE_SHADERS_GLSL)
-COGL_FEATURE_FUNCTION (GLhandleARB, glCreateProgramObject,
+COGL_FEATURE_BEGIN (shaders_glsl, 2, 0,
+                    "\0",
+                    "\0",
+                    COGL_FEATURE_SHADERS_GLSL,
+                    0)
+COGL_FEATURE_FUNCTION (GLuint, glCreateProgram,
                        (void))
-COGL_FEATURE_FUNCTION (GLhandleARB, glCreateShaderObject,
+COGL_FEATURE_FUNCTION (GLuint, glCreateShader,
                        (GLenum                shaderType))
 COGL_FEATURE_FUNCTION (void, glShaderSource,
-                       (GLhandleARB           shaderObj,
+                       (GLuint                shader,
                         GLsizei               count,
-                        const GLcharARB*     *string,
+                        const GLchar        **string,
                         const GLint          *length))
 COGL_FEATURE_FUNCTION (void, glCompileShader,
-                       (GLhandleARB           shaderObj))
-COGL_FEATURE_FUNCTION (void, glAttachObject,
-                       (GLhandleARB           containerObj,
-                        GLhandleARB           obj))
+                       (GLuint                shader))
+COGL_FEATURE_FUNCTION (void, glDeleteShader,
+                       (GLuint                shader))
+COGL_FEATURE_FUNCTION (void, glAttachShader,
+                       (GLuint                program,
+                        GLuint                shader))
 COGL_FEATURE_FUNCTION (void, glLinkProgram,
-                       (GLhandleARB           programObj))
-COGL_FEATURE_FUNCTION (void, glUseProgramObject,
-                       (GLhandleARB           programObj))
+                       (GLuint                program))
+COGL_FEATURE_FUNCTION (void, glUseProgram,
+                       (GLuint                program))
 COGL_FEATURE_FUNCTION (GLint, glGetUniformLocation,
-                       (GLhandleARB           programObj,
-                        const GLcharARB      *name))
-COGL_FEATURE_FUNCTION (void, glDeleteObject,
-                       (GLhandleARB           obj))
-COGL_FEATURE_FUNCTION (void, glGetInfoLog,
-                       (GLhandleARB           obj,
+                       (GLuint                program,
+                        const GLchar         *name))
+COGL_FEATURE_FUNCTION (void, glDeleteProgram,
+                       (GLuint                program))
+COGL_FEATURE_FUNCTION (void, glGetShaderInfoLog,
+                       (GLuint                shader,
                         GLsizei               maxLength,
                         GLsizei              *length,
-                        GLcharARB            *infoLog))
-COGL_FEATURE_FUNCTION (void, glGetObjectParameteriv,
-                       (GLhandleARB           obj,
+                        GLchar               *infoLog))
+COGL_FEATURE_FUNCTION (void, glGetShaderiv,
+                       (GLuint                shader,
                         GLenum                pname,
                         GLint                *params))
 
@@ -247,7 +284,8 @@ COGL_FEATURE_END ()
 COGL_FEATURE_BEGIN (vbos, 1, 5,
                     "ARB\0",
                     "vertex_buffer_object\0",
-                    COGL_FEATURE_VBOS)
+                    COGL_FEATURE_VBOS,
+                    0)
 COGL_FEATURE_FUNCTION (void, glGenBuffers,
                        (GLuint		 n,
                         GLuint		*buffers))
@@ -274,6 +312,13 @@ COGL_FEATURE_FUNCTION (void, glDeleteBuffers,
                         const GLuint		*buffers))
 COGL_FEATURE_END ()
 
+COGL_FEATURE_BEGIN (texture_rectangle, 255, 255,
+                    "ARB\0",
+                    "texture_rectangle\0",
+                    COGL_FEATURE_TEXTURE_RECTANGLE,
+                    0)
+COGL_FEATURE_END ()
+
 /* Cogl requires OpenGL 1.2 so we assume these functions are always
    available and don't bother setting any feature flags. We still have
    to fetch the function pointers though because under Windows you can
@@ -281,6 +326,7 @@ COGL_FEATURE_END ()
 COGL_FEATURE_BEGIN (in_1_2, 1, 2,
                     "\0",
                     "\0",
+                    0,
                     0)
 COGL_FEATURE_FUNCTION (void, glDrawRangeElements,
                        (GLenum                mode,
@@ -298,11 +344,33 @@ COGL_FEATURE_FUNCTION (void, glBlendColor,
                         GLclampf              alpha))
 COGL_FEATURE_END ()
 
+COGL_FEATURE_BEGIN (texture_3d, 1, 2,
+                    "\0",
+                    "\0",
+                    COGL_FEATURE_TEXTURE_3D,
+                    0)
+COGL_FEATURE_FUNCTION (void, glTexImage3D,
+                       (GLenum target, GLint level,
+                        GLint internalFormat,
+                        GLsizei width, GLsizei height,
+                        GLsizei depth, GLint border,
+                        GLenum format, GLenum type,
+                        const GLvoid *pixels))
+COGL_FEATURE_FUNCTION (void, glTexSubImage3D,
+                       (GLenum target, GLint level,
+                        GLint xoffset, GLint yoffset,
+                        GLint zoffset, GLsizei width,
+                        GLsizei height, GLsizei depth,
+                        GLenum format,
+                        GLenum type, const GLvoid *pixels))
+COGL_FEATURE_END ()
+
 /* Available in GL 1.3 or the multitexture extension. These are
    required */
 COGL_FEATURE_BEGIN (multitexture, 1, 3,
                     "ARB\0",
                     "multitexture\0",
+                    0,
                     0)
 COGL_FEATURE_FUNCTION (void, glActiveTexture,
                        (GLenum                texture))
@@ -314,6 +382,7 @@ COGL_FEATURE_END ()
 COGL_FEATURE_BEGIN (blend_func_separate, 1, 4,
                     "EXT\0",
                     "blend_func_separate\0",
+                    0,
                     0)
 COGL_FEATURE_FUNCTION (void, glBlendFuncSeparate,
                        (GLenum                srcRGB,
@@ -326,8 +395,16 @@ COGL_FEATURE_END ()
 COGL_FEATURE_BEGIN (blend_equation_separate, 2, 0,
                     "EXT\0",
                     "blend_equation_separate\0",
+                    0,
                     0)
 COGL_FEATURE_FUNCTION (void, glBlendEquationSeparate,
                        (GLenum                modeRGB,
                         GLenum                modeAlpha))
 COGL_FEATURE_END ()
+
+COGL_FEATURE_BEGIN (point_sprites, 2, 0,
+                    "ARB\0",
+                    "point_sprite\0",
+                    COGL_FEATURE_POINT_SPRITE,
+                    0)
+COGL_FEATURE_END ()
