@@ -192,7 +192,7 @@ clutter_stage_win32_resize (ClutterStageWindow *stage_window,
         }
 
       CLUTTER_SET_PRIVATE_FLAGS (stage_win32->wrapper,
-                                 CLUTTER_ACTOR_SYNC_MATRICES);
+                                 CLUTTER_SYNC_MATRICES);
     }
 }
 
@@ -222,7 +222,7 @@ _clutter_stage_win32_update_cursor (ClutterStageWin32 *stage_win32)
   HCURSOR cursor;
 
   if (stage_win32->is_cursor_visible)
-    cursor = (HCURSOR) GetClassLongPtrW (stage_win32->hwnd, GCL_HCURSOR);
+    cursor = (HCURSOR) GetClassLongPtrW (stage_win32->hwnd, GCLP_HCURSOR);
   else
     {
       ClutterBackend *backend = clutter_get_default_backend ();
@@ -349,7 +349,7 @@ clutter_stage_win32_set_fullscreen (ClutterStageWindow *stage_window,
         }
 
       CLUTTER_SET_PRIVATE_FLAGS (stage_win32->wrapper,
-				 CLUTTER_ACTOR_SYNC_MATRICES);
+				 CLUTTER_SYNC_MATRICES);
     }
 
   /* Report the state change */
@@ -553,8 +553,6 @@ clutter_stage_win32_init (ClutterStageWin32 *stage)
   stage->wtitle = NULL;
   stage->is_cursor_visible = TRUE;
   stage->wrapper = NULL;
-  	 
-  CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_IS_TOPLEVEL);
 }
 
 static void
